 /*
 * Copyright 2008 GigaSpaces Technologies Ltd. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS," WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY AND 
 * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT. GIGASPACES WILL NOT 
 * BE LIABLE FOR ANY DAMAGE OR LOSS IN CONNECTION WITH THE SOFTWARE.
 */

package com.j_spaces.examples.compress;

import com.j_spaces.core.IJSpace;
import com.j_spaces.core.client.SpaceFinder;
import com.j_spaces.core.client.FinderException;
import com.j_spaces.examples.compress.tasks.MemCheckTask;
import net.jini.core.entry.Entry;
import net.jini.core.entry.UnusableEntryException;
import net.jini.core.lease.Lease;
import net.jini.core.transaction.TransactionException;
import org.apache.commons.math.random.RandomDataImpl;
import java.util.List;
import java.util.ArrayList;

public class CompressMain {

    private IJSpace _space;
    private RandomDataImpl _randomData; 
    Progress progress = new Progress();
    public static void main(String[] args) {
        try{
            CompressMain cm = new CompressMain(args[0]);

            Integer dec = new Integer(args[1]);
            if(dec == null){
                System.exit(1);
            }
            switch (dec){
                case 0:
                    cm.testFunctionality();
                    break;
                case 1:
                    cm.testSimpleEntry();
                    break;
                case 3:
                    cm.testBinaryFormatEntry();
                    break;
                default:
                    System.exit(1);
            }
            System.exit(0);
        }catch(FinderException e){
            e.printStackTrace();
            System.exit(1);
        }catch(Exception ee){
            ee.printStackTrace();
            System.exit(1);
        }
    }


    public CompressMain(String url) throws FinderException {
        _space = (IJSpace) SpaceFinder.find(url);
        _randomData = new RandomDataImpl();
    }

    private void testFunctionality() throws Exception{
        SimpleEntry testSE = generateSimpleEntry(500);
        _space.write(testSE, null, Lease.FOREVER);
        SimpleEntry templateSE = new SimpleEntry();
        templateSE._queryField = new Integer(500);
        SimpleEntry resSE = (SimpleEntry)_space.read(templateSE, null, 0);
        
       // System.out.println("\nSimpleEntry\n-----------\nFieldA:"+resSE._stringFieldA + 
       // 		"\nFieldB:"+resSE._stringFieldB +
       //				"\nFieldC:"+resSE._stringFieldC);

        BinaryFormatEntry testBFE = generateBinaryFormatEntry(500);
        _space.write(testBFE, null, Lease.FOREVER);
        BinaryFormatEntry templateBFE = new BinaryFormatEntry();
        templateBFE.set_queryField (new Integer(500));
        BinaryFormatEntry resBFE = (BinaryFormatEntry)_space.read(templateBFE, null, 0);
        resBFE.unpack();
       // System.out.println("\nBinaryFormatEntry\n-----------\nFieldA:"+resBFE.getStringFieldA() + 
        //		"\nFieldB:"+resBFE.getStringFieldB() +
	//			"\nFieldC:"+resBFE.getStringFieldC());
    }

    private void testSimpleEntry() throws TransactionException, java.rmi.RemoteException,
            UnusableEntryException, InterruptedException {
        long  timeBefore = 0, timeAfter = 0;
        double avgWriteMultiple = 0, avgWrite = 0, avgRead = 0, avgReadMultiple = 0, avgTake = 0, avgTakeMultiple = 0,
                counter = 0;

        System.out.println("About to start testing Memory footprint and throughput for SimpleEntry " +
        "\nThis may take few minutes");
        _space.clean();
        long memoryBefore = memCheck();

        avgWriteMultiple = 250/(writeMultipleTest(1 , progress)/1000);
        long memoryAfter = memCheck();
        System.out.println("\nwrite test...");
        for(int i = 1 ; i < 10001 ; i++){
            SimpleEntry wse = generateSimpleEntry(i + 1000);
            timeBefore = System.currentTimeMillis();
            _space.write(wse, null, Lease.FOREVER);
            timeAfter = System.currentTimeMillis();
            counter += (timeAfter - timeBefore);
            
            if (i % 100 ==0)
            	progress.next();
        }
        avgWrite = 10000/(counter/1000);
        counter = 0;
        System.out.println("\nreadMultiple test...");

        for(int j = 1 ; j < 251 ; j++){
            SimpleEntry readMultipleTemplate = new SimpleEntry();
            readMultipleTemplate._queryField = new Integer(j);

            timeBefore = System.currentTimeMillis();
            Object res[] = _space.readMultiple(readMultipleTemplate, null, 1000);
            timeAfter = System.currentTimeMillis();
            counter += (timeAfter - timeBefore);
            if (res.length ==0) throw new RuntimeException("readMultiple rturn empty result");
            if (j % 100 ==0)
            	progress.next();
            
        }
        avgReadMultiple = 250/(counter/1000);
        counter = 0;
        System.out.println("\nread test...");

        for(int k = 1 ; k < 10001 ; k++){
            SimpleEntry readTemplate = new SimpleEntry();
            readTemplate._queryField = new Integer(k + 1000);
            timeBefore = System.currentTimeMillis();
            _space.read(readTemplate, null, 1000);
            timeAfter = System.currentTimeMillis();
            counter += (timeAfter - timeBefore);

            if (k % 100 ==0)
            	progress.next();
            
        }
        avgRead = 10000/(counter/1000);
        counter = 0;
        System.out.println("\ntake test...");

        for(int h = 1 ; h < 10001 ; h++){
            SimpleEntry takeTemplate = new SimpleEntry();
            takeTemplate._queryField = new Integer(h + 1000);
            timeBefore = System.currentTimeMillis();
            _space.take(takeTemplate, null, 1000);
            timeAfter = System.currentTimeMillis();
            counter += (timeAfter - timeBefore);
            if (h % 100 ==0)
            	progress.next();
            
        }
        avgTake = 10000/(counter/1000);
        counter = 0;
        System.out.println("\ntakeMultiple test...");

        for(int g = 1 ; g < 251 ; g++){
            SimpleEntry takeMultipleTemplate = new SimpleEntry();
            takeMultipleTemplate._queryField = new Integer(g);
            timeBefore = System.currentTimeMillis();
            Object res[]= _space.takeMultiple(takeMultipleTemplate, null, 1000);
            timeAfter = System.currentTimeMillis();
            counter += (timeAfter - timeBefore);
            if (res.length ==0) throw new RuntimeException("takeMultiple rturn empty result");
        }
        avgTakeMultiple = 250/(counter/1000);
        printResult("SimpleEntry", memoryAfter , memoryBefore , avgWrite , avgRead , avgTake, avgWriteMultiple , avgReadMultiple , avgTakeMultiple);

    }

    static public void printResult(String entryType, long  memoryAfter , long memoryBefore , double avgWrite , double avgRead , double avgTake, double avgWriteMultiple , double avgReadMultiple , double avgTakeMultiple)
    {
        System.out.println("\nThe results for :" + entryType);
        System.out.println("Average " + entryType + " size = " + (memoryAfter - memoryBefore)/250000);
        System.out.println(entryType + " - Average writeMultiple (1000 entries batch) TP = "+ (int)(avgWriteMultiple*1000) +" oper/sec");
        System.out.println(entryType + " - Average write TP = " + (int)avgWrite+" oper/sec");
        System.out.println(entryType + " - Average readMultiple  (1000 entries batch) TP = "
                + (int)(avgReadMultiple * 1000)+" oper/sec");
        System.out.println(entryType+ " - Average read TP = " + (int)avgRead +" oper/sec");
        System.out.println(entryType+ " - Average takeMultiple (1000 entries batch) TP = "  + (int)(avgTakeMultiple*1000)+" oper/sec");
        System.out.println(entryType + " - Average take TP = " + (int)avgTake+" oper/sec");
    }


    private void testBinaryFormatEntry() throws TransactionException, java.rmi.RemoteException,
        UnusableEntryException, InterruptedException{
        long timeBefore = 0, timeAfter = 0;
        double avgWriteMultiple = 0, avgWrite = 0,
                avgRead = 0, avgReadMultiple = 0, avgTake = 0, avgTakeMultiple = 0,
                counter = 0;

        System.out.println("About to start testing Memory footprint and throughput for BinaryFormatEntry " +
        "\nThis may take few minutes");
        _space.clean();

        long memoryBefore = memCheck();
        avgWriteMultiple = 250/(writeMultipleTest(3,progress)/1000);
        long memoryAfter = memCheck();
        System.out.println("\nwrite test...");

        for(int i = 1 ; i < 10001 ; i++){
            BinaryFormatEntry writeTemplate = generateBinaryFormatEntry(i + 1000);
            timeBefore = System.currentTimeMillis();
            _space.write(writeTemplate, null, Lease.FOREVER);
            timeAfter = System.currentTimeMillis();
            counter += (timeAfter - timeBefore);
            if (i % 100 ==0)
                progress.next();
            
        }
        avgWrite = 10000/(counter/1000);
        counter = 0;
        System.out.println("\nreadMultiple test...");

        for(int j = 1 ; j < 251 ; j++){
            BinaryFormatEntry readMultipleTemplate = new BinaryFormatEntry();
            readMultipleTemplate.set_queryField (new Integer(j));
            timeBefore = System.currentTimeMillis();
            Object res[] = _space.readMultiple(readMultipleTemplate, null, 1000);
            timeAfter = System.currentTimeMillis();
            counter += (timeAfter - timeBefore);
            if (res.length ==0) throw new RuntimeException("readMultiple rturn emptry result");
        }
        avgReadMultiple = 250/(counter/1000);
        counter = 0;
        System.out.println("\nread test...");

        for(int k = 1 ; k < 10001 ; k++){
            BinaryFormatEntry readTemplate = new BinaryFormatEntry();
            readTemplate.set_queryField (new Integer(k + 1000));
            timeBefore = System.currentTimeMillis();
            _space.read(readTemplate, null, 1000);
            timeAfter = System.currentTimeMillis();
            counter += (timeAfter - timeBefore);
            if (k % 100 ==0)
            	progress.next();
        }
        avgRead = 10000/(counter/1000);
        counter = 0;

        System.out.println("\ntake test...");
        for(int h = 1 ; h < 10001 ; h++){
            BinaryFormatEntry takeTemplate = new BinaryFormatEntry();
            takeTemplate.set_queryField (new Integer(h + 1000));
            timeBefore = System.currentTimeMillis();
            _space.take(takeTemplate, null, 1000);
            timeAfter = System.currentTimeMillis();
            counter += (timeAfter - timeBefore);
            if (h % 100 ==0)
            	progress.next();
        }
        avgTake = 10000/(counter/1000);
        counter = 0;
        System.out.println("\ntakeMultiple test...");

        for(int g = 1 ; g < 251 ; g++){
            BinaryFormatEntry takeMultipleTemplate = new BinaryFormatEntry();
            takeMultipleTemplate.set_queryField (new Integer(g));
            timeBefore = System.currentTimeMillis();
            Object res[] = _space.takeMultiple(takeMultipleTemplate, null, 1000);
            timeAfter = System.currentTimeMillis();
            counter += (timeAfter - timeBefore);
            if (res.length ==0) throw new RuntimeException("takeMultiple rturn emptry result");
        }
        avgTakeMultiple = 250/(counter/1000);

        printResult("BinaryFormatEntry", memoryAfter , memoryBefore , avgWrite , avgRead , avgTake, avgWriteMultiple , avgReadMultiple , avgTakeMultiple);

    }
    private long memCheck(){
    	System.out.println("\nChecking space memory...");
        progress.startAutoProgress();
        MemCheckTask mct = new MemCheckTask();
        mct._routing = 1;
        long res = 0;
        Entry[] ret;
        try{
            ret = _space.readMultiple(mct, null, Integer.MAX_VALUE);
            for(int i = 0 ; i < ret.length ; i++){
                MemCheckTask obj =  (MemCheckTask)ret[i];
                res += ((Long)obj._result);
            }
        }catch (Exception e){
            return 0;
        }
        progress.stopAutoProgress();
    	System.out.println("\nEnd Checking space memory");
        return res;
    }
    

    private double writeMultipleTest(int type , Progress prog) throws TransactionException, java.rmi.RemoteException{
    	long timeBefore = 0, timeAfter = 0;
        double counter = 0;
        System.out.println("writeMultiple test...");
        for(int i = 1 ; i < 251 ; i++){
            Object[] writeMArr = getWriteList(type, i);
            timeBefore = System.currentTimeMillis();
            _space.writeMultiple(writeMArr, null, Lease.FOREVER);
            prog.next();
            timeAfter = System.currentTimeMillis();
            counter += (timeAfter - timeBefore);
        }

        return counter;
    }


    private int getRandomNumber(int base, int max) {
        return _randomData.nextSecureInt(base, max);
    }

    private Object[] getWriteList(int type, int id){

        Object[] retArr = null;
        switch(type){
            case 1:
                retArr = generateSimpleEntryList(id);
                break;
            case 3:
                retArr = generateBinaryFormatEntryList(id);
                break;
            default:
                break;
        }
        return retArr;
    }

    private SimpleEntry[] generateSimpleEntryList(int id){
        List<SimpleEntry> simpleEntryList = new ArrayList<SimpleEntry>();
        for(int j = 0 ; j < 1000 ; j++){
            SimpleEntry se = generateSimpleEntry(id);
            simpleEntryList.add(se);
        }

        return simpleEntryList.toArray(new SimpleEntry[simpleEntryList.size()]);
    }

    private SimpleEntry generateSimpleEntry(int id){
        return new SimpleEntry(id, 
		      getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    String.valueOf(getRandomNumber(1, 1000)),
                    String.valueOf(getRandomNumber(1, 1000)),
                    String.valueOf(getRandomNumber(1, 1000)),
                    String.valueOf(getRandomNumber(1, 1000)),
		      getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    String.valueOf(getRandomNumber(1, 1000)),
                    String.valueOf(getRandomNumber(1, 1000)),
                    String.valueOf(getRandomNumber(1, 1000)),
                    String.valueOf(getRandomNumber(1, 1000)),
		      getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    String.valueOf(getRandomNumber(1, 1000)),
                    String.valueOf(getRandomNumber(1, 1000)),
                    String.valueOf(getRandomNumber(1, 1000)),
                    String.valueOf(getRandomNumber(1, 1000)));
    }


    private BinaryFormatEntry[] generateBinaryFormatEntryList(int id){
        List<BinaryFormatEntry> binaryFormatEntryList = new ArrayList<BinaryFormatEntry>();
        for(int j = 0 ; j < 1000 ; j++){
            BinaryFormatEntry bfe = generateBinaryFormatEntry(id);
            binaryFormatEntryList.add(bfe);
        }

        return binaryFormatEntryList.toArray(new BinaryFormatEntry[binaryFormatEntryList.size()]);
    }

    private BinaryFormatEntry generateBinaryFormatEntry(int id){
        BinaryFormatEntry bfe = new BinaryFormatEntry(id, 
		      getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    String.valueOf(getRandomNumber(1, 1000)),
                    String.valueOf(getRandomNumber(1, 1000)),
                    String.valueOf(getRandomNumber(1, 1000)),
                    String.valueOf(getRandomNumber(1, 1000)),
		      getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    String.valueOf(getRandomNumber(1, 1000)),
                    String.valueOf(getRandomNumber(1, 1000)),
                    String.valueOf(getRandomNumber(1, 1000)),
                    String.valueOf(getRandomNumber(1, 1000)),
		      getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    getRandomNumber(1, 1000),
                    String.valueOf(getRandomNumber(1, 1000)),
                    String.valueOf(getRandomNumber(1, 1000)),
                    String.valueOf(getRandomNumber(1, 1000)),
                    String.valueOf(getRandomNumber(1, 1000)));

        bfe.pack();
        return bfe;
    }
}
