package com.j_spaces.example.customfifonotifications;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicInteger;

public class SizeConcurrentHashMap<K, V> extends ConcurrentHashMap<K, V>
{
   final private AtomicInteger _size = new AtomicInteger();

   public SizeConcurrentHashMap (int initialCapacity, float loadFactor, int concurrencyLevel)
   {
      super(initialCapacity, loadFactor, concurrencyLevel);
   }

   public SizeConcurrentHashMap (int initialCapacity)
   {
      super(initialCapacity);
   }

   public SizeConcurrentHashMap ()
   {
      super();
   }

   public SizeConcurrentHashMap(Map<K, V> t)
   {
      super(t);
   }
   
   @Override
   final public V putIfAbsent( K key, V value)
   {
      V o = super.putIfAbsent( key, value);
      if( o == null)
      {
         _size.incrementAndGet();
      }
      return o;
   }
   
   @Override
   final public V put( K key, V value)
   {
      V o = super.put( key, value);
      if( o == null)
      {
         _size.incrementAndGet();
      }
      return o;
   }
   
   @Override
   final public V remove( Object key)
   {
      V o = super.remove( key);
      if( o != null)
      {
         _size.decrementAndGet();
      }
      return o;
   }
   
   @Override
   final public boolean remove( Object key, Object value)
   {
      if( super.remove( key, value))
      {
         _size.decrementAndGet();
         return true;
      }
      return false;
   }
   
   @Override
   final public int size()
   {
      return _size.get();
   }
   
   @Override
   final public void clear()
   {
      super.clear();
      _size.set( super.size());
   }

}
