package com.j_spaces.examples.parentchild;

import java.rmi.RemoteException;
import java.util.HashSet;
import java.util.Set;

import org.openspaces.core.GigaSpace;
import org.openspaces.core.GigaSpaceConfigurer;
import org.openspaces.core.space.UrlSpaceConfigurer;

import net.jini.core.entry.UnusableEntryException;
import net.jini.core.transaction.TransactionException;
import com.j_spaces.core.client.ClientUIDHandler;
import com.j_spaces.core.client.ExternalEntry;

public class ParentChildMain {

	static GigaSpace space;
	static int max_graphs = 1000;
	static int matching_graphs = 5;

	public static void main(String[] args) {
		try {

			 UrlSpaceConfigurer urlSpaceConfigurer = new UrlSpaceConfigurer("/./mySpace");
			 space = new GigaSpaceConfigurer(urlSpaceConfigurer.space()).gigaSpace();

			 	space.clean();

				System.out.println("warm up....");
				for (int i = 0; i < 2000; i++) {
					Parent parent = new Parent(i +"");
					
					//space.write(parent);
					space.writeMultiple(new Object[]{parent});
					space.read(parent);
					space.readMultiple(parent , Integer.MAX_VALUE);
					space.take(parent);
					
					Child child  = new Child ("a", null);
					space.writeMultiple(new Object[]{child});
					space.read(child);
					space.readMultiple(child , Integer.MAX_VALUE);
					space.take(child);
					
					if (i% 100 == 1)
						System.out.print(".");
					if (i% (100 * 80) == 1)
						System.out.println();
					
				}

			System.out.println();
			System.out.println("Write " + max_graphs+ " parent/child graphs.\nWe will have 2 types of graphs:" +
					"\n" +  matching_graphs+" graphs that got the following values for the child objects:0,1,2,3,4,5,6,7,8,9" +
					"\nand another "  +(max_graphs - matching_graphs) + "  graphs with the following values for the child objects:0,10,20,30,40,50,60,70,80,90");
			
			//space.clear(new Parent());
			go();
			space.clean();
			go();
			space.clean();
			go();
			space.clean();
			go();
			
		} catch (Exception e) {
			e.printStackTrace();
		}		}
		static public void go() throws Exception
		{
			for (int i = 0; i < max_graphs; i++) {
				Parent parent = new Parent(i +"");
				Child childs[] = new Child[10];
				parent.childs_uid = new String[10];
				for (int j = 0; j < 10; j++) {
					childs[j] = new Child(i + "_" + j,	parent.getId());
					if (i% (max_graphs/matching_graphs) ==0)
						childs[j].field = new Long(j);
					else
						childs[j].field = new Long(j * 10);
						
					parent.childs_uid[j] = childs[j].getId();
				}
				space.write(parent);
				space.writeMultiple(childs);
			}

			System.out.println("Lets find all the parent object that got 2 child object with values 1 and 2 - effectively join");
			System.out.println("We need to make sure that both child objects have the same parent object");
			Long startTime = System.nanoTime();
			Object childs_results1[] = getChildsbyValue(new Long(1));
			Set set1 = getParentUIDsSet(childs_results1);

			Object childs_results2[] = getChildsbyValue(new Long(2));
			Set set2 = getParentUIDsSet(childs_results2);
			
			Set resultSet = AND(set1 , set2);

			Parent parents[] = getParentsfromUIDs(resultSet);
			Long endTime = System.nanoTime();
			System.out.println("-----------------------------------------------------------------");
			System.out.println(" --->> Found " + parents.length +" matching Parent objects in " + (double)(((double)endTime - (double)startTime)/1000000) + " ms");
			
			for (int i = 0; i < parents.length; i++) {
				System.out.println("Found Parent Object:" + parents[i]
						+ " - ID:" + parents[i].getId() + " - His children objects are:\n"
						+ parents[i]._getChildsDetails(space));
			}
	}

	static public Object[] getChildsbyValue(Long value) throws RemoteException, TransactionException, UnusableEntryException {
		Child child_template = new Child();
		child_template.field = value;
		return space.readMultiple(child_template, Integer.MAX_VALUE);
	}
	
	static public Set getParentUIDsSet(Object entries[]) {
		HashSet result = new HashSet();
		for (int i = 0; i < entries.length; i++) {
			result.add(((Child) entries[i]).parentUID);
		}
		return result;
	}

	static public Parent[] getParentsfromUIDs(Set uids) throws UnusableEntryException, RemoteException, TransactionException
	{
		Parent[] ret = null;
		String uids_str[] = new String [uids.size()]; 
		System.arraycopy(uids.toArray(),0,uids_str,0,uids_str.length);
		String parentClassName = Parent.class.getName();
		for (int i=0;i<uids_str.length;i++)
		{
			uids_str[i] = ClientUIDHandler.createUIDFromName(uids_str[i] , parentClassName );
		}
		ExternalEntry child_uids_template = new ExternalEntry(uids_str);
		Object parent_results[] = space.readMultiple(child_uids_template,Integer.MAX_VALUE);
		ret = new Parent[parent_results.length];
		for (int i=0;i<parent_results.length ; i++)
		{
			ret[i] = (Parent) ((ExternalEntry)parent_results[i]).getObject(space.getSpace());
		}
		return ret;
	}
	
	// find intersection between 2 sets with UIDs
	static public Set AND(Set set1, Set set2) {
		HashSet result = new HashSet(set1);
		//Iterator keys = set1.iterator();
		result.retainAll(set2);
		/*
		while (keys.hasNext()) {
			String key = (String) keys.next();
			if (set2.contains(key)) {
				result.add(key);
			}
		}
		*/
		return result;
	}
}
