package net.sf.hibernate.gigacache;

import java.util.Properties;
import com.j_spaces.core.IJSpace;
import com.j_spaces.core.client.*;
import com.j_spaces.map.*;
import net.jini.core.lease.Lease;
import org.hibernate.cache.Cache;
import org.hibernate.cache.CacheProvider;
import org.hibernate.cache.Timestamper;

public class GigaSpacesCacheProvider implements CacheProvider {
	static IMap cache = null;

	static boolean isUsingLocalCache = true;

	// space url prefix contains:
	// Protocol://[host]:[port]/[container_name]/[space_name]
	// e.g. "java://[host]:10098/[container_name]/JavaSpaces"
	private static String _spaceURLPrefix = System.getProperty("spaceurl")
			.toString();

	// e.g. "?useLocalCache=false&create=true&versioned=false"
	private static String _spaceURLOptions = System.getProperty(
			"spaceurloptions").toString();

	/**
	 * 
	 * @see net.sf.hibernate.cache.CacheProvider#buildCache(java.lang.String,
	 *      java.util.Properties)
	 */
	public Cache buildCache(String regionName, Properties properties)
			throws CacheException {
		try {
			// Find a space proxy according to the space url prefix +
			// the additional space url options
			// the _spaceURL should look like:
			// java://[host]:10098/[container_name]/[space
			// name]?useLocalCache&create=true&versioned=false
			String _spaceURL = _spaceURLPrefix + _spaceURLOptions;

			System.out.println("Getting Space proxy:" + _spaceURL
					+ " For Region:" + regionName);
			if (cache == null) {
				cache = (IMap) CacheFinder.find(_spaceURL);
				if (_spaceURL.toLowerCase().indexOf(
						SpaceURL.USE_LOCAL_CACHE.toLowerCase()) > 0) {
					isUsingLocalCache = true;
					System.out.println("Local Cache Used!");
				} else// support a case where a Local Cache is used.
				{
					isUsingLocalCache = false;
					System.out.println("No Local Cache Usage!");
				}
				System.out.println("Getting Space proxy - OK");
			}
		} catch (Exception e) {
			System.out.println("Getting Space proxy - ERROR!");
			e.printStackTrace();
		}
		return new GigaHibernateCache(cache, isUsingLocalCache, regionName);
	}

	public long nextTimestamp() {
		return Timestamper.next();
	}

	public boolean isMinimalPutsEnabledByDefault() {
		return true;
	}

	public void start(Properties properties) throws CacheException {
	}

	public void stop() {
	}
}