
package com.jmxutil;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.lang.management.ManagementFactory;
import java.lang.management.MemoryMXBean;
import java.lang.management.MemoryUsage;
import java.lang.management.RuntimeMXBean;
import java.lang.management.ThreadMXBean;
import java.util.Iterator;
import java.util.Set;

import javax.management.MBeanInfo;
import javax.management.MBeanNotificationInfo;
import javax.management.MBeanOperationInfo;
import javax.management.MBeanParameterInfo;
import javax.management.MBeanServerConnection;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;

import com.j_spaces.core.filters.FilterOperationCodes;
import com.j_spaces.core.filters.StatisticsContext;

/**
 * @author Yogesh.Parchure
 * <p>February 2008
 * @version 1.0
 * <p>Parameters to this class are the JMX URL for the space, result file name, 
 * statistics interval time in milliseconds.
 */
public class JMXSpaceStats {
	
	public static void main(String[] args) {
		PrintWriter pw=null;
		if (args.length == 0) 
		{
			showUsage();
			System.exit(1);
		}
		if (! (args[0].startsWith("service:jmx:rmi://")))
		{
			invalidURL();
			System.exit(1);
		}
		if (args[1] == null || args[1].length() == 0)
		{
			echo("Result file not specified");
			System.exit(1);
		}
		File resultFile = new File(args[1]);
		try {
			pw = new PrintWriter(new FileOutputStream(resultFile),true);
		} catch (FileNotFoundException e1) {
			// TODO Auto-generated catch block
			echo("Caught Exception while opening file " + args[1]);
			e1.printStackTrace();
			System.exit(1);
		}
		long sleeptime = 5000;
		if (args[2] != null && args[2].length() != 0)
		{
			try {
				sleeptime = Long.parseLong(args[2]);
			}
			catch (NumberFormatException nfe)
			{
				sleeptime = 5000;
			}
		}
		try {
			// Create an RMI connector client and
			// connect it to the RMI connector server
			//
			echo("\nCreate an RMI connector client and " +
			"connect it to the RMI connector server");
			JMXServiceURL url = new JMXServiceURL(args[0]);
			JMXConnector jmxc = JMXConnectorFactory.connect(url, null);
			
			// Get an MBeanServerConnection
			//
			echo("\nGet an MBeanServerConnection");
			MBeanServerConnection mbsc = jmxc.getMBeanServerConnection();
		
			// Get domains from MBeanServer
			//
			echo("\nDomains:");
			String domains[] = mbsc.getDomains();
			for (int i = 0; i < domains.length; i++) {
				echo("\tDomain[" + i + "] = " + domains[i]);
			}
			
			// Get MBeanServer's default domain
			//
			String domain = mbsc.getDefaultDomain();
			echo("Default Domain is " + domain);
			
		
			// Get MBean count
			//
			echo("\nMBean count = " + mbsc.getMBeanCount());
			
			// Query MBean names
			//
			echo("\nQuery MBeanServer MBeans:");
			ObjectName gsmbeanName=null;
			Set names = mbsc.queryNames(null, null);
			for (Iterator i = names.iterator(); i.hasNext(); ) {
				ObjectName testObj = (ObjectName)i.next();
				echo("\tObjectName = " + testObj);
				if (testObj.toString().startsWith("com.gigaspaces:type=JavaSpace,name="))
					gsmbeanName = testObj;
			}
			if (null == gsmbeanName)
			{
				echo("GigaSpaces Space MBean not found. Exiting.");
				System.exit(1);
			}
			// Get a MBean proxy for RuntimeMXBean interface
			RuntimeMXBean proxy = (RuntimeMXBean)
			ManagementFactory.newPlatformMXBeanProxy(mbsc,
					ManagementFactory.RUNTIME_MXBEAN_NAME,
					RuntimeMXBean.class);
			
			echo("Runtime JVM Name: " + proxy.getVmName() + " JVM Vendor: " + proxy.getVmVendor());
			
			ThreadMXBean threadproxy = (ThreadMXBean)
			ManagementFactory.newPlatformMXBeanProxy(mbsc,
					ManagementFactory.THREAD_MXBEAN_NAME,
					ThreadMXBean.class);
			echo("ThreadProxy.getThreadCount()= " + threadproxy.getThreadCount() + 
					"\nThreadProxy.getTotalStartedThreadCount()=" + threadproxy.getTotalStartedThreadCount());
			
			MemoryMXBean memoryproxy = (MemoryMXBean)
			ManagementFactory.newPlatformMXBeanProxy(mbsc,
					ManagementFactory.MEMORY_MXBEAN_NAME,
					MemoryMXBean.class);
			MemoryUsage memused = memoryproxy.getHeapMemoryUsage();
			echo("Heap: Commited " + memused.getCommitted()
					+ " init " + memused.getInit()
					+ " Max " + memused.getMax() 
					+ " used " + memused.getUsed());
			MBeanInfo gsmbeaninfo = mbsc.getMBeanInfo(gsmbeanName);
			MBeanOperationInfo[] gsmbeanops = gsmbeaninfo.getOperations();
			MBeanNotificationInfo[] gsmbeannotify = gsmbeaninfo.getNotifications();
			MBeanParameterInfo[] gsmbeanGetStatisticsparms;
			String[] gs_getstats_return=null;
			for(int i=0;i<gsmbeanops.length;i++ )
			{
				/* echo("Operation " + i + ":" + gsmbeanops[i].getDescription() +
						":" + gsmbeanops[i].getName() +
						":" + gsmbeanops[i].getReturnType());
				*/
				// get the method signature for getStatistics
				if (gsmbeanops[i].getName().equals("getStatistics"))
				{
					gsmbeanGetStatisticsparms =	gsmbeanops[i].getSignature();
					gs_getstats_return= new String[gsmbeanGetStatisticsparms.length];
					for(int j=0;j<gsmbeanGetStatisticsparms.length;j++)
					{
						gs_getstats_return[j] = gsmbeanGetStatisticsparms[j].getType();
						// echo("OpSignatureReturn:" + j +":" + gs_getstats_return[j] );
					}
				}
			}
			/**
			for(int i=0;i<gsmbeannotify.length;i++ )
			{
				echo("Notification " + i + ":" + gsmbeannotify[i].getDescription() +
						":" + gsmbeannotify[i].getName() );
			}
			*/
			pw.println("\n\nConnected to: " + gsmbeanName);
			pw.println("\n\nSytem.currentTimeMillis, jvm threadcount, jvm heap committed, jvm heap max, " +
					"jvm heap used,space read count, space write count, space take count");
			Object[] beforeReadFilters = new Object[] {new Integer(FilterOperationCodes.BEFORE_READ) };
			Object[] afterReadMultipleFilters = new Object[] {new Integer(FilterOperationCodes.AFTER_READ_MULTIPLE) };
			Object[] beforeWriteFilters = new Object[] {new Integer(FilterOperationCodes.BEFORE_WRITE) };
			Object[] beforeTakeFilters = new Object[] { new Integer(FilterOperationCodes.BEFORE_TAKE) };
			Object[] beforeTakeMultFilters = new Object[] { new Integer(FilterOperationCodes.BEFORE_TAKE_MULTIPLE) };
			
			for(int i=0;i<100;i++)
			{
				StatisticsContext readStat = (StatisticsContext)
				mbsc.invoke(gsmbeanName,"getStatistics", 
						beforeReadFilters,
						gs_getstats_return);
				StatisticsContext readMStat = (StatisticsContext)
				mbsc.invoke(gsmbeanName,"getStatistics",
						afterReadMultipleFilters,
						gs_getstats_return);

				StatisticsContext writeStat = (StatisticsContext)
				mbsc.invoke(gsmbeanName,"getStatistics",
						beforeWriteFilters,
						gs_getstats_return);

				StatisticsContext takeStat = (StatisticsContext)
				mbsc.invoke(gsmbeanName,"getStatistics",
						beforeTakeFilters,
						gs_getstats_return);
				
				StatisticsContext takeMStat = (StatisticsContext)
				mbsc.invoke(gsmbeanName,"getStatistics",
						beforeTakeMultFilters,
						gs_getstats_return);

				Thread.sleep(5000);
				memused = memoryproxy.getHeapMemoryUsage();
				pw.println(System.currentTimeMillis() + "," + threadproxy.getThreadCount() + "," +
						memused.getCommitted() + "," +
						memused.getMax() + "," +
						memused.getUsed() + "," +
						( 
							(readStat == null ? 0: readStat.getCurrentCount())
							+
						 (readMStat == null ? 0 : readMStat.getCurrentCount())
						) + "," +
						(writeStat == null ? 0 : writeStat.getCurrentCount())
						 + "," +
						 (
						 (takeStat == null ?0 :takeStat.getCurrentCount()) +
						 (takeMStat == null ?0 :takeMStat.getCurrentCount())
						 )
						);
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			pw.close();
		}
	}
	
	/**
	 * 
	 */
	private static void invalidURL() {
		// TODO Auto-generated method stub
		echo("jmx url must begin with 'service:jmx:rmi://' ");
	}
	
	/**
	 * 
	 */
	private static void showUsage() {
		// TODO Auto-generated method stub
		echo("SpacesJVMStatistics jmxurl");
	}
	
	private static void echo(String msg) {
		System.out.println(msg);
	}
	
	private static void sleep(int millis) {
		try {
			Thread.sleep(millis);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}
	
	private static void waitForEnterPressed() {
		try {
			echo("\nPress <Enter> to continue...");
			System.in.read();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
}
