/**
 * GigaSpaces Technologies Examples
 * Title:        Utility.java
 * Description:  Space-Admin functions
 * Copyright:    Copyright (c) GigaSpaces Technology Ltd.
 * Company:      GigaSpaces Technologies
 * 
 */
package com.j_spaces.examples.xmlreplicationfilter;

import com.j_spaces.core.IJSpace;
import com.j_spaces.core.admin.IRemoteJSpaceAdmin;
import com.j_spaces.core.admin.SpaceConfig;
import com.j_spaces.core.client.SpaceFinder;
import com.j_spaces.core.cluster.ClusterPolicy;

import java.util.*;

public class Utility {

	public static void waitOnActiveCluster(IJSpace space, long ms) throws Exception {
		if (isClusteredSpace(space)) {
			while (true) {
				if (isClusteredActive(space) == false) {
					System.out.println("(UtilityClass-Output) Wait to establish Cluster-Connection...");
					Thread.sleep(ms);
				} else {
					System.out.println("(UtilityClass-Output) Cluster Active!");
					break;
				}
			}
		} else { System.out.println("(UtilityClass-Output) None Clustered Space !"); }

	}

	public static boolean isClusteredActive(IJSpace space) throws Exception {
		try {
			// retrieve IRemoteJSpaceAdmin from the space. This interface contains all the 
			// administrative methods that GigaSpaces provides to control the JavaSpaces service. 
			IRemoteJSpaceAdmin admin = (IRemoteJSpaceAdmin) space.getAdmin();
			ClusterPolicy clusterPolicy = admin.getClusterPolicy();
			if (clusterPolicy.m_ReplicationPolicy != null) {
				// getReplicationStatus: returns the replication member-status of all 
				// the replication groups. 
				Object replStatus[] = admin.getReplicationStatus();
				String[] peers = (String[]) replStatus[0];
				int[] status = (int[]) replStatus[1];
				for (int i = 0; i < status.length; i++) {
					System.out.println("(UtilityClass-Output) node:"
									+ peers[i]
									+ " - Active:"
									+ (status[i] == IRemoteJSpaceAdmin.REPLICATION_STATUS_ACTIVE));
					if (status[i] != IRemoteJSpaceAdmin.REPLICATION_STATUS_ACTIVE) {
						return false;
					}
				}
			} else {
				List membersURLs = null;
				if (clusterPolicy.m_LoadBalancingPolicy != null)
					membersURLs = clusterPolicy.m_LoadBalancingPolicy.loadBalanceGroupMembersURLs;
				else if (clusterPolicy.m_FailOverPolicy != null)
					membersURLs = clusterPolicy.m_FailOverPolicy.failOverGroupMembersURLs;
				else
					return false;

				if (membersURLs != null) {
					for (int i = 0; i < membersURLs.size(); i++) {
						IJSpace _space = (IJSpace) SpaceFinder.find(membersURLs.get(i).toString());
						if (_space == null)
							return false;
					}
				} else
					return false;
			}
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}

	public static boolean isClusteredSpace(IJSpace space) {
		boolean isClustered = false;

		try {
			// retrieve IRemoteJSpaceAdmin from the space. This interface contains all the 
			// administrative methods that GigaSpaces provides to control the JavaSpaces service. 
			IRemoteJSpaceAdmin admin = (IRemoteJSpaceAdmin) space.getAdmin();
			SpaceConfig config = admin.getConfig();
			if (config != null)
				isClustered = Boolean.valueOf( config.isClustered() ).booleanValue();

		} catch (Throwable e) {
			e.printStackTrace();
			isClustered = false;
		}
		return isClustered;
	}
}
