/**
 * 
 * Title:        	JiniSpaceAccessor
 * Description:  	Used for getting GigaSpaces space proxy using jini discovery LookupLocator.
 * 					
 */
package com.gigaspaces.examples.tutorials.plainjavaspaces;

import java.rmi.RMISecurityManager;

import net.jini.core.discovery.LookupLocator;
import net.jini.core.entry.Entry;
import net.jini.core.lookup.ServiceRegistrar;
import net.jini.core.lookup.ServiceTemplate;
import net.jini.lookup.entry.Name;
import net.jini.space.JavaSpace;

public class JiniSpaceAccessor {

	/**
	 *	findSpace
	 *	Finds a JavaSpaces service and returns a space proxy object connected to it.
	 * 
	 * @param spaceName - Holds the space name (in this example- "mySpace")
	 * @return JavaSpace
	 * @throws Exception
	 */ 
	static public JavaSpace findSpace(String spaceName) throws Exception {

		//	Get a RMI security manger, if doesn't exist set a new one.
		//	The security policy is passed to this security manager from the running script "run.bat"
		//	using the -Djava.security.policy flag.
		if ( System.getSecurityManager() == null )
			System.setSecurityManager( new RMISecurityManager() );
		
		//	Create a template to lookup the JavaSpaces service.
		Class [] classes = new Class[]{net.jini.space.JavaSpace.class};
		Name sn = new Name( spaceName );
		ServiceTemplate tmpl = new ServiceTemplate(null,classes,new Entry[] { sn } );

		//	Locate the JavaSpaces service and create a JavaSpace proxy attached to it.
		LookupLocator locator = new LookupLocator("jini://localhost");
		ServiceRegistrar sr = locator.getRegistrar();
		JavaSpace space =  (JavaSpace)sr.lookup(tmpl);
		
		return space;
	}
}
